package PureMessage::Socket;

use strict;
use Socket;
use Exporter;
use base qw(DynaLoader);

our $VERSION = '4.100';

sub import {
    my $pkg = shift;
    my $callpkg = caller;
    Exporter::export 'Socket', $callpkg, @_;
}

PureMessage::Socket->bootstrap($VERSION);

sub accept {
    my $o = shift;
    my $pkg = shift;
    my $paddr;
    my $client = $o->_accept($paddr);
    return unless defined $client;
    bless $client, $pkg ? ref($pkg) || $pkg : ref($o);
    wantarray ? ($client, $paddr) : $client;
}

sub DESTROY {
    my $o = shift;
    $o->close;
    $o->_free;
}

sub connected {
    my $o = shift;
    $o->peername;
}

sub autoflush { 1 }

*read = *sysread;
*write = *syswrite;

sub scheme { 'ANY' }

package PureMessage::Socket::INET;
use base qw(PureMessage::Socket);
use Socket;

my $proto_tcp = getprotobyname('tcp');

sub new {
    my ($cls, %opts) = @_;

    # Check for unimplemented features
    if ($opts{Type} and $opts{Type} != SOCK_STREAM) {
	die "Unsupported Type '$opts{Type}': must be SOCK_STREAM"
    }

    my $self;
    if (exists $opts{Listen}) {
	my $iaddr = INADDR_ANY;
	$iaddr = inet_aton($opts{LocalAddr} || $opts{LocalHost})
	    if $opts{LocalAddr} || $opts{LocalHost};
	my $paddr = sockaddr_in($opts{LocalPort}, $iaddr);
	$self = _listener(
	    $paddr,
	    $proto_tcp,
	    $opts{Reuse} || $opts{ReuseAddr} || 0,
	    $opts{Listen} || 1024
	);
    }
    else {
	my $iaddr = inet_aton($opts{PeerAddr} || $opts{PeerHost} || '127.0.0.1');
	my $paddr = sockaddr_in($opts{PeerPort}, $iaddr);
	$self = _peer($paddr, $proto_tcp);
    }
    return unless defined $self;
    bless $self, ref($cls) || $cls; # bless into appropriate subclass
}

sub scheme { 'INET' }

sub sockaddr {
    my $o = shift;
    my $name = $o->sockname;
    $name ? (sockaddr_in($name))[1] : undef;
}

sub sockport {
    my $o = shift;
    my $name = $o->sockname;
    $name ? (sockaddr_in($name))[0] : undef;
}

sub sockhost {
    my $o = shift;
    my $addr = $o->sockaddr;
    $addr ? inet_ntoa($addr) : undef;
}

sub peeraddr {
    my $o = shift;
    my $name = $o->peername;
    $name ? (sockaddr_in($name))[1] : undef;
}

sub peerport {
    my $o = shift;
    my $name = $o->peername;
    $name ? (sockaddr_in($name))[0] : undef;
}

sub peerhost {
    my $o = shift;
    my $addr = $o->peeraddr;
    $addr ? inet_ntoa($addr) : undef;
}

package PureMessage::Socket::UNIX;
use base qw(PureMessage::Socket);
use Socket;

sub new {
    my ($cls, %opts) = @_;

    # Check for unimplemented features
    if ($opts{Type} and $opts{Type} != SOCK_STREAM) {
	die "Unsupported Type '$opts{Type}': must be SOCK_STREAM"
    }

    my $self;
    if (exists $opts{Listen}) {
	my $paddr = sockaddr_un($opts{Local});
	$self = _listener($paddr, $opts{Listen} || 1024);
    }
    else {
	my $paddr = sockaddr_un($opts{Peer});
	$self = _peer($paddr);
    }
    return unless defined $self;
    bless $self, ref($cls) || $cls; # bless into appropriate subclass
}

sub scheme { 'UNIX' }
sub hostpath {
    my $o = shift;
    my $name = $o->sockname;
    $name ? (sockaddr_un($name))[0] : undef;
}

sub peerpath {
    my $o = shift;
    my $name = $o->peername;
    $name ? (sockaddr_un($name))[0] : undef;
}

1;

__END__

=head1 NAME

PureMessage::Socket - sockets without stdio file handles

=head1 SYNOPSIS

   use PureMessage::Socket;

   my $sock = PureMessage::Socket::INET->new(
       LocalAddr => '127.0.0.1',
       LocalPort => '500',
       Type      => SOCK_STREAM,
       Reuse     => 1,
       Listen    => 1024,
   );
   
   my $client = $sock->accept;
   # OR
   my ($client, $paddr) = $sock->accept;

   my $written = $client->write($message, $length, $offset);
   my $read = $client->read($buffer, $length, $offset);

=head1 DESCRIPTION

IO::Socket works fine for many applications. Unfortunately, perl internally
uses two stdio filehandles for each socket (one each for reading and writing). On
Solaris, it turns out that there is a limit of 256 such handles, so using two
per socket is too many. This library is written entirely in C, and uses no
stdio filehandles. As a consequence, all file IO is unbuffered.

PureMessage::Socket itself does not have a new() method. You must create sockets
using one of PureMessage::Socket::INET or PureMessage::Socket::UNIX.

All methods return C<undef> and set C<$!> on failure.

=head1 PureMessage::Socket

The following methods are available on all sockets:

=over 4

=item accept()

   $client = $sock->accept($pkg);
   $client = $sock->accept;
   ($client, $paddr) = $sock->accept;

Calls accept() on the socket and return a new socket, created in the same
class as the listen socket, unless $pkg is specified. In a scalar context the
new socket is returned, or undef upon failure. In an array context a
two-element list is returned containing the new socket and the peer address.
The list will be empty upon failure.

=item autoflush()

   $client->autoflush($flush);
   $client->autoflush();

Because PureMessage::Socket is written entirely at the file descriptor layor,
autoflush is I<always> enabled.

=item blocking()

   $o->blocking( $block);
   $o->blocking();

If called with an argument, blocking() will turn on non-blocking IO if
C<$block> is false, and turn it off if C<$block> is true. The previous value
of blocking will be returned.

If called without an argument, returns the current blocking-ness.

=item close()

   $o->close();

Closes the socket. Subsequent method calls will probably not work very well.
All read()s and write()s will return 0, for example.

=item connected()

   $o->connected();

If the socket is in a connected state then the peer address is returned,
otherwise undef will be returned.

=item fileno()

   $o->fileno();

Returns the underlying file number of the socket. Returns C<-1> after close()
has been called, since the original file number may be associated with a
different file.

=item peername()

Returns the packed sockaddr address of the other end of the socket connection.

=item read()

Synonym for sysread().

=item sockname()

Returns the packed sockaddr address of this end of the socket connection.
Useful if there are several different IP addresses the connection might have
come in on.

=item sysread()

   $o->sysread($buffer, $length, $offset);
   $o->sysread($buffer, $length);

Reads from the socket, storing bytes in C<$buffer>. Returns the number of
bytes actually read. WARNING: this method allocates enough memory for the
entire read to succeed, so trying to read a huge block of data will consume a
huge block of memory, even if only 1 byte is available on the socket.

=item syswrite()

   $o->syswrite($buffer, $length, $offset);
   $o->syswrite($buffer, $length);
   $o->syswrite($buffer);

Writes the contents of C<$buffer> to the socket in much the same way as the
builtin syswrite() function. Returns the number of bytes successfully written.

=item write()

Synonym for syswrite().

=back

=head1 PureMessage::Socket::INET

This section describes the methods specific to PureMessage::Socket::INET.

=over 4

=item new()

    $sock = PureMessage::Socket::INET->new(%opts)

Creates a new Internet-domain socket.

The following options are available:

=over 4

=item Listen

The flag which indicates whether to create a listening socket. If this key
exists, a listening socket is created. If a value is provided as well, it is
used as the number of client connections to queue before refusing connections.
The default is 1024.

=item LocalAddr

Local host bind address. Must be a hostname or dot-separated IP address. The
C<host:port> syntax is not supported. If omitted when a listening socket is
created, then C<IPADDR_ANY> is used, otherwise C<127.0.0.1> is used.

=item LocalHost

Synonym for C<LocalAddr>.

=item LocalPort

Local host bind port. Required when creating a listening socket. Must be an
integer.

=item PeerAddr

Remote host address. Has the same requirements and defaults as C<LocalAddr>,
but used when connecting to a remote host.

=item PeerHost

Synonym for C<PeerAddr>.

=item PeerPort

Remote port. Required when connecting to a remote host. Must be an integer.

=item Reuse

Sets SO_REUSEADDR before binding the socket.

=item Type

If provided, must be SOCK_STREAM, the only type of socket currently supported.

=back

If successful, returns a new socket. Returns C<undef> on
error; check C<$!> for the error message.

=item sockaddr()

Returns the local internet address the socket is bound to.

=item sockport()

Returns the port number that the socket is using on the local host.

=item sockhost()

Returns the local internet address the socket is bound to, formatted in a text
form xx.xx.xx.xx.

=item peeraddr()

Returns the internet address for the socket on the peer host. Returns C<undef>
if this socket is not connected to another endpoint.

=item peerport()

Returns the port number for the socket on the peer host.

=item peerhost()

Returns the internet address for the socket on the peer host, formatted in a
text form xx.xx.xx.xx.

=item scheme()

Returns the string "INET".

=back

=head1 PureMessage::Socket::UNIX

This section describes the methods specific to PureMessage::Socket::UNIX.

=over 4

=item new()

    $sock = PureMessage::Socket::UNIX->new(%opts)

Creates a Unix-domain socket.

The following options are recognized:

=over 4

=item Listen

The flag which indicates whether to create a listening socket. If this key
exists, a listening socket is created. If a value is provided as well, it is
used as the number of client connections to queue before refusing connections.
The default is 1024.

=item Local

The fifo (a full Unix path) to which to bind.

=item Peer

The fifo (a full Unix path) to which to connect.

=item Type

If provided, must be SOCK_STREAM, the only type of socket currently supported.

=back

If successful, returns a new socket. Returns C<undef> on error; check C<$!>
for the error message.

=item hostpath()

Returns the pathname to the fifo at the local end.

=item peerpath()

Returns the pathname to the fifo at the peer end. Interestingly, this always
returns C<undef> on Linux. This behaviour can be duplicated with IO::Socket,
or even in C, so it's a Linux "feature", not a bug.

=item scheme()

Returns the string "UNIX".

=back

=head1 COPYRIGHT

Copyright (C) 2002, ActiveState Corporation. All Rights Reserved.

=cut
